@@ -1,4 +1,4 @@
-# -.- coding: utf-8 -.-
+# -*- coding: utf-8 -*-
 # ###################################################
 # Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
@@ -25,9 +25,11 @@ import os
 import os.path
 import platform
 import subprocess
+from pathlib import Path
+from typing import List, Optional
 
 from horizons.ext.enum import Enum
-from horizons.ext.typing import Optional
+from horizons.util.platform import get_old_user_game_directory, get_user_game_directories
 
 
 """This file keeps track of the constants that are used in Unknown Horizons.
@@ -35,6 +37,7 @@ NOTE: Using magic constants in code is generally a bad style, so avoid where
 possible and instead import the proper classes of this file.
 """
 
+
 def get_git_version():
 	"""Function gets latest revision of the working copy.
 	It only works in git repositories, and is actually a hack.
@@ -43,7 +46,7 @@ def get_git_version():
 		from run_uh import get_content_dir_parent_path
 		uh_path = get_content_dir_parent_path()
 	except ImportError:
-		return u"<unknown>"
+		return "<unknown>"
 
 	# Try git describe
 	try:
@@ -54,31 +57,36 @@ def get_git_version():
 		# Note that this uses glob patterns, not regular expressions.
 		TAG_STRUCTURE = "20[0-9][0-9].[0-9]*"
 		describe = [git, "describe", "--tags", "--match", TAG_STRUCTURE]
-		return unicode(subprocess.check_output(describe, cwd=uh_path))
+		git_string = subprocess.check_output(describe, cwd=uh_path, universal_newlines=True).rstrip('\n')
+		return git_string
 	except (subprocess.CalledProcessError, RuntimeError):
 		pass
 
 	# Read current HEAD out of .git manually
 	try:
-		git_head_path = os.path.join(uh_path, '.git', 'HEAD')
-		if os.path.exists(git_head_path):
-			head = open(git_head_path).readline().strip().partition(' ')
+		git_head_path = Path(uh_path, '.git', 'HEAD')
+		if git_head_path.exists():
+			with git_head_path.open() as f:
+				head = f.readline().strip().partition(' ')
 			if head[2]:
-				head_file = os.path.join(uh_path, '.git', head[2])
+				head_file = Path(uh_path, '.git', head[2])
 			else:
 				head_file = git_head_path
-			if os.path.exists(head_file):
-				return unicode(open(head_file).readline().strip()[0:7])
+
+			if head_file.exists():
+				with head_file.open() as f:
+					return str(f.readline().strip()[0:7])
 	except ImportError:
 		pass
 
 	# Try gitversion.txt
 	try:
-		return unicode(open(os.path.join("content", "packages", "gitversion.txt")).read())
+		with open(os.path.join("content", "packages", "gitversion.txt")) as f:
+			return f.read()
 	except IOError:
 		pass
 
-	return u"<unknown>"
+	return "<unknown>"
 
 
 ##Versioning
@@ -86,23 +94,24 @@ class VERSION:
 	RELEASE_NAME    = "Unknown Horizons %s"
 	#RELEASE_VERSION = get_git_version()
 	# change for release:
-	IS_DEV_VERSION = True
-	RELEASE_VERSION = u'2017.2'
+	IS_DEV_VERSION = False
+	RELEASE_VERSION = u'2019.1'
 
 	REQUIRED_FIFE_MAJOR_VERSION = 0
 	REQUIRED_FIFE_MINOR_VERSION = 4
-	REQUIRED_FIFE_PATCH_VERSION = 0
+	REQUIRED_FIFE_PATCH_VERSION = 2
 
 	REQUIRED_FIFE_VERSION = (REQUIRED_FIFE_MAJOR_VERSION, REQUIRED_FIFE_MINOR_VERSION, REQUIRED_FIFE_PATCH_VERSION)
 
 	## +=1 this if you changed the savegame "api"
-	SAVEGAMEREVISION = 76
-	SAVEGAME_LEAST_UPGRADABLE_REVISION = 48
+	SAVEGAMEREVISION = 77
+	SAVEGAME_LEAST_UPGRADABLE_REVISION = 76
 
 	@staticmethod
 	def string():
 		return VERSION.RELEASE_NAME % VERSION.RELEASE_VERSION
 
+
 ## WORLD
 class UNITS:
 	# ./development/print_db_data.py unit
@@ -120,7 +129,7 @@ class UNITS:
 
 	WILD_ANIMAL          = 1000013
 	HUNTER_COLLECTOR     = 1000014
-	FARM_ANIMAL_COLLECTOR= 1000015
+	FARM_ANIMAL_COLLECTOR = 1000015
 	USABLE_FISHER_BOAT   = 1000016
 
 	FRIGATE              = 1000020
@@ -134,6 +143,7 @@ class UNITS:
 
 	DIFFERENCE_BUILDING_UNIT_ID = 1000000
 
+
 class BUILDINGS:
 	# ./development/print_db_data.py building
 	WAREHOUSE        =  1
@@ -199,14 +209,19 @@ class BUILDINGS:
 	WINERY           = 65
 
 	WEAPONSMITH      = 66
-	CANNONFOUNDRY    = 67
+	CANNON_FOUNDRY   = 67
 
 	BREWERY          = 68
 	HOP_FIELD        = 69
 
 	STONE_DEPOSIT    = 70
 
-	BARRIER	         = 71
+	BARRIER          = 71
+
+	AMBIENT          = 72
+
+	SALINE           = 86
+	PUBLIC_BATH      = 87
 
 	EXPAND_RANGE = (WAREHOUSE, STORAGE, LOOKOUT)
 
@@ -218,20 +233,21 @@ class BUILDINGS:
 		# you need to sort this before iterating via sorted, since order is important here.
 		action_offset_dict = {
 		# Direct connections
-		  'a' : ( 0, -1),
-		  'b' : (+1,  0),
-		  'c' : ( 0, +1),
-		  'd' : (-1,  0),
+		  'a': (0, -1),
+		  'b': (+1, 0),
+		  'c': (0, +1),
+		  'd': (-1, 0),
 		# Remote connections
-		  'e' : (+1, -1),
-		  'f' : (+1, +1),
-		  'g' : (-1, +1),
-		  'h' : (-1, -1),
+		  'e': (+1, -1),
+		  'f': (+1, +1),
+		  'g': (-1, +1),
+		  'h': (-1, -1),
 		}
 
 	class BUILD:
 		MAX_BUILDING_SHIP_DISTANCE = 5 # max distance ship-building when building from ship
 
+
 class RES:
 	# ./development/print_db_data.py res
 	GOLD             =  1
@@ -330,18 +346,20 @@ class RES:
 	SOCIETY          = GOLD # 93
 	FAITH_2          = GOLD # 94
 	EDUCATION_2      = GOLD # 95
-	HYGIENE          = GOLD # 96
+	HYGIENE          = 96
 	RECREATION       = GOLD # 97
 	BLACKDEATH       = 98
 	FIRE             = 99
 	# 92-99 reserved for services
 
+
 class WEAPONS:
 	CANNON = RES.CANNON
 	SWORD  = RES.SWORD
 
 	DEFAULT_FIGHTING_SHIP_WEAPONS_NUM = 7
 
+
 class GROUND:
 	DEFAULT_LAND = (3, "straight", 45)
 	SAND = (6, "straight", 45)
@@ -390,19 +408,24 @@ class GROUND:
 	DEEP_WATER_SOUTHWEST1 = (2, "curve_out", 45)
 	DEEP_WATER_NORTHWEST1 = (2, "curve_out", 315)
 
+
 class ACTION_SETS:
 	DEFAULT_ANIMATION_LENGTH = 500
 	DEFAULT_WEIGHT = 10
 
+
 class GAME_SPEED:
 	TICKS_PER_SECOND = 16
 	TICK_RATES = [] # type: List[int]
 
+
 GAME_SPEED.TICK_RATES = [int(i * GAME_SPEED.TICKS_PER_SECOND) for i in (0.5, 1, 2, 3, 4, 6, 8, 11, 20)]
 
+
 class COLORS:
 	BLACK = 9
 
+
 class VIEW:
 	ZOOM_MAX = 1.5
 	ZOOM_MIN = 0.25
@@ -413,6 +436,7 @@ class VIEW:
 	TILT = -60
 	AUTOSCROLL_WIDTH = 10
 
+
 ## The Production States available in the game sorted by importance from least
 ## to most important
 class PRODUCTION:
@@ -422,6 +446,7 @@ class PRODUCTION:
 	# NOTE: 'none' is not used by an actual production, just for a producer
 	STATISTICAL_WINDOW = 1000 # How many latest ticks are relevant for keeping track of how busy a production is
 
+
 class PRODUCTIONLINES:
 	HUKER = 15
 	# will get added later
@@ -441,33 +466,39 @@ class GAME:
 	# exit after on tick MAX_TICKS (disabled by setting to None)
 	MAX_TICKS = None # type: Optional[int]
 
+
 # Map related constants
 class MAP:
 	PADDING = 10 # extra usable water around the map edges
 	BORDER = 30 # extra unusable water around the padding (to keep the black void at bay)
 
+
 class GUI:
 	CITYINFO_UPDATE_DELAY = 2 # seconds
 	DEFAULT_EXCHANGE_AMOUNT = 50  # tons
 
+
 # Editor
 class EDITOR:
 	MIN_BRUSH_SIZE = 1
 	MAX_BRUSH_SIZE = 3
 	DEFAULT_BRUSH_SIZE = 1
 
+
 # Messagewidget and Logbook
 class MESSAGES:
 	CUSTOM_MSG_SHOW_DELAY = 6 # delay between messages when passing more than one
 	CUSTOM_MSG_VISIBLE_FOR = 90 # after this time the msg gets removed from screen
 	LOGBOOK_DEFAULT_DELAY = 1 # delay between condition fulfilled and logbook popping up
 
+
 # AI values read from the command line; use the values below unless overridden by the CLI or the GUI
 class AI:
 	HIGHLIGHT_PLANS = False # whether to show the AI players' plans on the map
 	HIGHLIGHT_COMBAT = False # whether to show the AI players' combat ranges around each unit
 	HUMAN_AI = False # whether the human player is controlled by the AI
 
+
 class TRADER: # check resource values: ./development/print_db_data.py res
 	TILES_PER_TRADER = 100 # create one ship per 100 tiles
 	SETTLEMENTS_PER_SHIP = 2 # the settlement : ship ratio
@@ -482,6 +513,7 @@ class TRADER: # check resource values: ./development/print_db_data.py res
 	SELL_AMOUNT_MIN = 2
 	SELL_AMOUNT_MAX = 10
 
+
 # Taxes and Restrictions
 class TIER:
 	NATURE = 0
@@ -494,13 +526,15 @@ class TIER:
 
 	LOWEST = SAILORS
 	HIGHEST = ARISTOCRATS
-	CURRENT_MAX = CITIZENS
+	CURRENT_MAX = MERCHANTS
+
 
 class SETTLER:
 	TAX_SETTINGS_MIN = 0.5
 	TAX_SETTINGS_MAX = 1.5
 	TAX_SETTINGS_STEP = 0.1
 
+
 class WILD_ANIMAL:
 	HEALTH_INIT_VALUE = 50 # animals start with this value
 	HEALTH_INCREASE_ON_FEEDING = 8 # health increases by this value on feeding
@@ -510,12 +544,14 @@ class WILD_ANIMAL:
 	FOOD_AVAILABLE_ON_START = 0.5 # probability that a tree has wild animal food in the beginning
 	POPULATION_INIT_RATIO = 15 # every N-th tree gets an animal in the beginning
 
+
 class COLLECTORS:
 	DEFAULT_WORK_DURATION = 16 # how many ticks collectors pretend to work at target
 	DEFAULT_WAIT_TICKS = 32 # how long collectors wait before again looking for a job
 	DEFAULT_STORAGE_SIZE = 8
 	STATISTICAL_WINDOW = 1000 # How many latest ticks are relevant for calculating how busy a collector is
 
+
 class STORAGE:
 	DEFAULT_STORAGE_SIZE = 30 # Our usual inventorys are 30 tons big
 
@@ -524,33 +560,22 @@ class STORAGE:
 	SHIP_TOTAL_STORAGE = 120
 	SHIP_TOTAL_SLOTS_NUMBER = 4
 
+	# The maximum number of items you can set in a trade route slot or warehouse trade slot
+	# This was actually only added to make sure the number wouldn't overflow the amount slider
+	# If more items can be traded than pixels in the slider it is impossible to accurately select an amount
+	# See https://github.com/unknown-horizons/unknown-horizons/issues/1095
+	ITEMS_PER_TRADE_SLOT = 50
+
+
 ## ENGINE
 class LAYERS:
 	WATER = 0
 	GROUND = 1
-	FIELDS = 2
+	FIELDS = 2 # ironically doesn't apply to fields anymore...
 	OBJECTS = 3
 
 	NUM = 4 # number of layers
 
-## PATHS
-# workaround, so it can be used to create paths within PATHS
-if 'UH_USER_DIR' in os.environ:
-	# Prefer the value from the environment. Used to override user dir when
-	# running GUI tests.
-	_user_dir = unicode(os.environ['UH_USER_DIR'], encoding='utf-8')
-elif platform.system() != "Windows":
-	_user_dir = os.path.join(os.path.expanduser('~'), '.unknown-horizons')
-else:
-	import ctypes.wintypes
-	buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
-	# get the My Documents folder into buf.value
-	ctypes.windll.shell32.SHGetFolderPathW(0, 5, 0, 0, buf)
-	my_games = os.path.join(buf.value, 'My Games')
-	if not os.path.exists(my_games):
-		os.makedirs(my_games)
-	_user_dir = os.path.join(my_games, 'unknown-horizons')
-
 
 class GFX:
 	BUILDING_OUTLINE_THRESHOLD = 96
@@ -565,18 +590,27 @@ class GFX:
 	# this is modified by the game starting process.
 	USE_ATLASES = False
 
+
+## PATHS
+
+_config_dir, _data_dir, _cache_dir = get_user_game_directories()
+
+
 class PATHS:
-	# paths in user dir
-	USER_DIR = _user_dir
-	LOG_DIR = os.path.join(USER_DIR, "log")
-	USER_MAPS_DIR = os.path.join(USER_DIR, "maps")
-	USER_CONFIG_FILE = os.path.join(USER_DIR, "settings.xml")
-	SCREENSHOT_DIR = os.path.join(USER_DIR, "screenshots")
-	DEFAULT_WINDOW_ICON_PATH = os.path.join("content", "gui", "images", "logos", "uh_32.png")
-	MAC_WINDOW_ICON_PATH = os.path.join("content", "gui", "icons", "Icon.icns")
-	ATLAS_METADATA_PATH = os.path.join(USER_DIR, "atlas-metadata.cache")
+	# paths in user directories
+	USER_CONFIG_FILE = os.path.join(_config_dir, "settings.xml")
+	USER_DATA_DIR = _data_dir
+	LOG_DIR = os.path.join(USER_DATA_DIR, "log")
+	USER_MAPS_DIR = os.path.join(USER_DATA_DIR, "maps")
+	SCREENSHOT_DIR = os.path.join(USER_DATA_DIR, "screenshots")
+	SAVEGAME_DIR = os.path.join(USER_DATA_DIR, "save")
+	CACHE_DIR = _cache_dir
+	ATLAS_METADATA_PATH = os.path.join(CACHE_DIR, "atlas-metadata.cache")
 
 	# paths relative to uh dir
+	DEFAULT_WINDOW_ICON_PATH = os.path.join("content", "gui", "images", "logos", "uh_32.png")
+	MAC_WINDOW_ICON_PATH = os.path.join("content", "gui", "icons", "Icon.icns")
+
 	ACTION_SETS_DIRECTORY = os.path.join("content", "gfx")
 	TILE_SETS_DIRECTORY = os.path.join("content", "gfx", "base")
 	SAVEGAME_TEMPLATE = os.path.join("content", "savegame_template.sql")
@@ -589,47 +623,60 @@ class PATHS:
 	SETTINGS_TEMPLATE_FILE = os.path.join("content", "settings-template.xml")
 	CONFIG_TEMPLATE_FILE = os.path.join("content", "settings-template.xml")
 
-
 	DB_FILES = tuple(os.path.join("content", i) for i in
 	                 ("game.sql", "balance.sql", "names.sql"))
 
-	ATLAS_SOURCE_DIRECTORIES = tuple(os.path.join("content/gfx", d)
+	ATLAS_SOURCE_DIRECTORIES = tuple(os.path.join("content", "gfx", d)
 	                                 for d in (
-	                                 "/base",
-	                                 "/buildings",
-	                                 "/misc",
-	                                 "/terrain",
-	                                 "/units",
+	                                 "base",
+	                                 "buildings",
+	                                 "misc",
+	                                 "terrain",
+	                                 "units",
 	                                ))
 
 	#voice paths
 	VOICE_DIR = os.path.join("content", "audio", "voice")
 	UH_LOGO_FILE = os.path.join("content", "gfx", "uh.png")
 
+
+_old_user_dir = get_old_user_game_directory()
+
+
+class OLDPATHS:
+	USER_DATA_DIR = _old_user_dir
+	USER_CONFIG_FILE = os.path.join(_old_user_dir, "settings.xml")
+	CACHE_DIR = _old_user_dir
+
+
 class SETTINGS:
 	UH_MODULE = "unknownhorizons"
 	FIFE_MODULE = "FIFE"
 	KEY_MODULE = "keys"
 	META_MODULE = "meta"
 
+
 class PLAYER:
 	STATS_UPDATE_FREQUENCY = GAME_SPEED.TICKS_PER_SECOND
 
+
 ## SINGLEPLAYER
 class SINGLEPLAYER:
 	FREEZE_PROTECTION = True
 	SEED = None # type: int
 
+
 ## MULTIPLAYER
 class MULTIPLAYER:
 	MAX_PLAYER_COUNT = 8
 
+
 class NETWORK:
-	SERVER_ADDRESS = "master.unknown-horizons.org"
+	SERVER_ADDRESS = "207.180.251.79"
 	# change port to 2022 for development server updated after UH commits
 	SERVER_PORT = 2002
 	CLIENT_ADDRESS = None # type: Optional[str]
-	UPDATE_FILE_URL = "http://updates.unknown-horizons.org/current_version.php"
+	UPDATE_FILE_URL = "http://unknown-horizons.org/current-version.json"
 
 
 ## TRANSLATIONS
@@ -641,7 +688,7 @@ class _LanguageNameDict(dict):
 		return self.get(key, [key])[1]
 
 	def get_by_value(self, value, english=False):
-		for code, (own, eng) in self.iteritems():
+		for code, (own, eng) in self.items():
 			if english and eng == value:
 				return code
 			elif not english and own == value:
@@ -650,56 +697,59 @@ class _LanguageNameDict(dict):
 
 
 LANGUAGENAMES = _LanguageNameDict({
-	""      : (u'System default', u''),
-	"af"    : (u'Afrikaans', u'Afrikaans'),
-	"bg"    : (u'Български', u'Bulgarian'),
-	"ca"    : (u'Català', u'Catalan'),
-	'ca@valencia' : (u'Català de València', u'Catalan (Valencia)'),
-	"cs"    : (u'Čeština', u'Czech'),
-	"da"    : (u'Danske', u'Danish'),
-	"de"    : (u'Deutsch', u'German'),
-	"en"    : (u'English', u'English'),
-	"eo"    : (u'Esperanto', u'Esperanto'),
-	"es"    : (u'Español', u'Spanish'),
-	"et"    : (u'Eesti', u'Estonian'),
-	"el"    : (u'Ελληνικά', u'Greek'),
-	"fi"    : (u'Suomi', u'Finnish'),
-	"fr"    : (u'Français', u'French'),
-	"frp"   : (u'Francoprovençâl', u'Franco-Provencal'),
-	"ga"    : (u'Gaeilge', u'Irish'),
-	"gl"    : (u'Galego', u'Galician'),
-	"hi"    : (u'मानक हिन्दी', u'Hindi'),
-	"hr"    : (u'Hrvatski', u'Croatian'),
-	"hu"    : (u'Magyar', u'Hungarian'),
-	"id"    : (u'Bahasa Indonesia', u'Indonesian'),
-	"it"    : (u'Italiano', u'Italian'),
-	"ja"    : (u'日本語', u'Japanese'),
-	"ko"    : (u'한국말/조선말', u'Korean'),
-	"lt"    : (u'Lietuvių', u'Lithuanian'),
-	"lv"    : (u'Latviešu', u'Latvian'),
-	"ml"    : (u'മലയാളം', u'Malayalam'),
-	"nb"    : (u'Bokmål', u'Norwegian'),
-	"nl"    : (u'Nederlands', u'Dutch'),
-	"pl"    : (u'Polski', u'Polish'),
-	"pt_BR" : (u'Português Br.', u'Brazilian Portuguese'),
-	"pt"    : (u'Português', u'Portuguese'),
-	"ro"    : (u'Română', u'Romanian'),
-	"ru"    : (u'Русский', u'Russian'),
-	"sk"    : (u'Slovenský', u'Slovak'),
-	"sl"    : (u'Slovenski', u'Slovenian'),
-	"sr"    : (u'Cрпски', u'Serbian'),
-	"sv"    : (u'Svenska', u'Swedish'),
-	"th"    : (u'ภาษาไทย', u'Thai'),
-	"tr"    : (u'Türkçe', u'Turkish'),
-	"uk"    : (u'Українська', u'Ukrainian'),
-	"vi"    : (u'Tiếng Việt', u'Vietnamese'),
-	"zh_CN" : (u'简化字', u'Simplified Chinese'),
-	"zh_TW" : (u'繁體字', u'Traditional Chinese'),
-	"zu"    : (u'IsiZulu', u'Zulu'),
+	""      : ('System default', ''),
+	"af"    : ('Afrikaans', 'Afrikaans'),
+	"ar"    : ('اَللُّغَةُ اَلْعَرَبِيَّة', 'Arabic'),
+	"bg"    : ('Български', 'Bulgarian'),
+	"ca"    : ('Català', 'Catalan'),
+	'ca@valencia' : ('Català de València', 'Catalan (Valencia)'),
+	"cs"    : ('Čeština', 'Czech'),
+	"da"    : ('Danske', 'Danish'),
+	"de"    : ('Deutsch', 'German'),
+	"en"    : ('English', 'English'),
+	"eo"    : ('Esperanto', 'Esperanto'),
+	"es"    : ('Español', 'Spanish'),
+	"et"    : ('Eesti', 'Estonian'),
+	"el"    : ('Ελληνικά', 'Greek'),
+	"fi"    : ('Suomi', 'Finnish'),
+	"fr"    : ('Français', 'French'),
+	"frp"   : ('Francoprovençâl', 'Franco-Provencal'),
+	"ga"    : ('Gaeilge', 'Irish'),
+	"gl"    : ('Galego', 'Galician'),
+	"hi"    : ('मानक हिन्दी', 'Hindi'),
+	"hr"    : ('Hrvatski', 'Croatian'),
+	"hu"    : ('Magyar', 'Hungarian'),
+	"id"    : ('Bahasa Indonesia', 'Indonesian'),
+	"it"    : ('Italiano', 'Italian'),
+	"ja"    : ('日本語', 'Japanese'),
+	"ko"    : ('한국말/조선말', 'Korean'),
+	"lt"    : ('Lietuvių', 'Lithuanian'),
+	"lv"    : ('Latviešu', 'Latvian'),
+	"ml"    : ('മലയാളം', 'Malayalam'),
+	"nb"    : ('Bokmål', 'Norwegian'),
+	"nl"    : ('Nederlands', 'Dutch'),
+	"pl"    : ('Polski', 'Polish'),
+	"pt_BR" : ('Português Br.', 'Brazilian Portuguese'),
+	"pt"    : ('Português', 'Portuguese'),
+	"ro"    : ('Română', 'Romanian'),
+	"ru"    : ('Русский', 'Russian'),
+	"sk"    : ('Slovenský', 'Slovak'),
+	"sl"    : ('Slovenski', 'Slovenian'),
+	"sr"    : ('Cрпски', 'Serbian'),
+	"sv"    : ('Svenska', 'Swedish'),
+	"th"    : ('ภาษาไทย', 'Thai'),
+	"tr"    : ('Türkçe', 'Turkish'),
+	"uk"    : ('Українська', 'Ukrainian'),
+	"vi"    : ('Tiếng Việt', 'Vietnamese'),
+	"zh_CN" : ('简化字', 'Simplified Chinese'),
+	"zh_Hant" : ('繁體字', 'Traditional Chinese'),
+	"zh_TW" : ('繁體字', 'Traditional Chinese'),
+	"zu"    : ('IsiZulu', 'Zulu'),
 })
 
 FONTDEFS = {
 	# "af"
+	# "ar"
 	"bg"    : 'libertine',
 	# "ca"
 	"ca@valencia" : 'libertine',
@@ -712,6 +762,7 @@ FONTDEFS = {
 	"el"    : 'libertine',
 	"fi"    : 'libertine',
 	"fr"    : 'libertine',
+	# "frp"
 	"ga"    : 'libertine',
 	"gl"    : 'libertine',
 	# "hi"
@@ -720,9 +771,10 @@ FONTDEFS = {
 	"id"    : 'libertine',
 	"it"    : 'libertine',
 	# "ja"
+	# "ko"
 	"lt"    : 'libertine',
 	"lv"    : 'libertine',
-	# "ko"
+	# "ml"
 	"nb"    : 'libertine',
 	"nl"    : 'libertine',
 	"pl"    : 'libertine',
@@ -739,9 +791,11 @@ FONTDEFS = {
 	"uk"    : 'libertine',
 	# "vi"
 	# "zh_CN"
+	# "zh_TW"
 	"zu"    : 'libertine',
 }
 
+
 class HOTKEYS:
 	DISPLAY_KEY = {
 		'MINUS': '-',
