@@ -22,6 +22,7 @@
 import logging
 import random
 import weakref
+from typing import TYPE_CHECKING, List
 
 from fife import fife
 
@@ -31,7 +32,6 @@ from horizons.command.sounds import PlaySound
 from horizons.component.selectablecomponent import SelectableBuildingComponent, SelectableComponent
 from horizons.constants import BUILDINGS, GFX
 from horizons.entities import Entities
-from horizons.ext.typing import TYPE_CHECKING
 from horizons.extscheduler import ExtScheduler
 from horizons.gui.mousetools.navigationtool import NavigationTool
 from horizons.gui.util import load_uh_widget
@@ -39,7 +39,6 @@ from horizons.i18n import gettext as T
 from horizons.messaging import (
 	PlayerInventoryUpdated, SettlementInventoryUpdated, SettlementRangeChanged, WorldObjectDeleted)
 from horizons.util.loaders.actionsetloader import ActionSetLoader
-from horizons.util.python import decorators
 from horizons.util.shapes import Point
 from horizons.util.worldobject import WorldObject
 
@@ -104,7 +103,7 @@ class BuildingTool(NavigationTool):
 	gui = None # type: Widget
 
 	def __init__(self, session, building, ship=None, build_related=None):
-		super(BuildingTool, self).__init__(session)
+		super().__init__(session)
 		assert not (ship and build_related)
 		self.renderer = self.session.view.renderer['InstanceRenderer']
 		self.ship = ship
@@ -126,7 +125,7 @@ class BuildingTool(NavigationTool):
 		if self.ship is not None:
 			self._build_logic = ShipBuildingToolLogic(ship)
 		elif build_related is not None:
-			self._build_logic = BuildRelatedBuildingToolLogic(self, weakref.ref(build_related) )
+			self._build_logic = BuildRelatedBuildingToolLogic(self, weakref.ref(build_related))
 		else:
 			self._build_logic = SettlementBuildingToolLogic(self)
 
@@ -140,7 +139,6 @@ class BuildingTool(NavigationTool):
 		SettlementInventoryUpdated.subscribe(self.update_preview)
 		PlayerInventoryUpdated.subscribe(self.update_preview)
 
-
 	def __init_selectable_component(self):
 		self.selectable_comp = SelectableBuildingComponent
 		try:
@@ -173,18 +171,18 @@ class BuildingTool(NavigationTool):
 
 		renderer = self.session.view.renderer['InstanceRenderer']
 		if tiles_to_check is None or new_buildings: # first run, check all
-			buildings_to_select = [ buildings_to_select
-			                        for settlement in self.session.world.settlements
-			                        if settlement.owner.is_local_player
-			                        for bid in related
-			                        for buildings_to_select in settlement.buildings_by_id[bid] ]
+			buildings_to_select = [buildings_to_select
+			                       for settlement in self.session.world.settlements
+			                       if settlement.owner.is_local_player
+			                       for bid in related
+			                       for buildings_to_select in settlement.buildings_by_id[bid]]
 
 			tiles = self.selectable_comp.select_many(buildings_to_select, renderer)
 			self._related_buildings_selected_tiles = frozenset(tiles)
 		else: # we don't need to check all
 			# duplicates filtered later
-			buildings_to_select = [ tile.object for tile in tiles_to_check if
-			                        tile.object is not None and tile.object.id in related ]
+			buildings_to_select = [tile.object for tile in tiles_to_check if
+			                       tile.object is not None and tile.object.id in related]
 			for tile in tiles_to_check:
 				# check if we need to recolor the tiles
 				if tile in self._related_buildings_selected_tiles:
@@ -216,27 +214,27 @@ class BuildingTool(NavigationTool):
 		ExtScheduler().rem_all_classinst_calls(self)
 		SettlementInventoryUpdated.discard(self.update_preview)
 		PlayerInventoryUpdated.discard(self.update_preview)
-		super(BuildingTool, self).remove()
+		super().remove()
 
 	def _on_worldobject_deleted(self, message):
 		# remove references to this object
 		self._related_buildings.discard(message.sender)
 		self._transparencified_instances = \
-		  set( i for i in self._transparencified_instances if
-		       i() is not None and int(i().getId()) != message.worldid )
-		check_building = lambda b : b.worldid != message.worldid
-		self._highlighted_buildings = set( tup for tup in self._highlighted_buildings if check_building(tup[0]) )
-		self._related_buildings = set( filter(check_building, self._related_buildings) )
+		  set(i for i in self._transparencified_instances if
+		      i() is not None and int(i().getId()) != message.worldid)
+		check_building = lambda b: b.worldid != message.worldid
+		self._highlighted_buildings = set(tup for tup in self._highlighted_buildings if check_building(tup[0]))
+		self._related_buildings = set(filter(check_building, self._related_buildings))
 
 	def load_gui(self):
 		if self.__class__.gui is None:
 			self.__class__.gui = load_uh_widget("place_building.xml")
 			self.__class__.gui.position_technique = "right-1:top+157"
-		self.__class__.gui.mapEvents({"rotate_left" : self.rotate_left,
+		self.__class__.gui.mapEvents({"rotate_left": self.rotate_left,
 		                              "rotate_right": self.rotate_right})
 		# set translated building name in gui
 		self.__class__.gui.findChild(name='headline').text = T('Build {building}').format(building=T(self._class.name))
-		self.__class__.gui.findChild(name='running_costs').text = unicode(self._class.running_costs)
+		self.__class__.gui.findChild(name='running_costs').text = str(self._class.running_costs)
 		head_box = self.__class__.gui.findChild(name='head_box')
 		head_box.adaptLayout() # recalculates size of new content
 		# calculate and set new center
@@ -258,7 +256,7 @@ class BuildingTool(NavigationTool):
 				action = action_option
 				break
 		else: # If no idle, idle_full or abcd animation found, use the first you find
-			action = action_sets[action_set].keys()[0]
+			action = list(action_sets[action_set].keys())[0]
 		rotation = (self.rotation + int(self.session.view.cam.getRotation()) - 45) % 360
 		image = sorted(action_sets[action_set][action][rotation].keys())[0]
 		if GFX.USE_ATLASES:
@@ -300,7 +298,7 @@ class BuildingTool(NavigationTool):
 		self.buildings = new_buildings
 		# resize list of action set ids to new buildings
 		self.buildings_action_set_ids = self.buildings_action_set_ids + ([None] * (len(self.buildings) - len(self.buildings_action_set_ids)))
-		self.buildings_action_set_ids = self.buildings_action_set_ids[ : len(self.buildings) ]
+		self.buildings_action_set_ids = self.buildings_action_set_ids[: len(self.buildings)]
 		# delete old infos
 		self.buildings_fife_instances.clear()
 		self.buildings_missing_resources.clear()
@@ -337,7 +335,6 @@ class BuildingTool(NavigationTool):
 				if settlement is None:
 					building.buildable = False
 
-
 			# check required resources
 			(enough_res, missing_res) = Build.check_resources(needed_resources, self._class.costs,
 			                                                  self.session.world.player, [settlement, self.ship])
@@ -362,7 +359,7 @@ class BuildingTool(NavigationTool):
 
 		self.session.ingame_gui.resource_overview.set_construction_mode(
 			self.ship if self.ship is not None else settlement,
-		  needed_resources
+			needed_resources
 		)
 		self._add_listeners(self.ship if self.ship is not None else settlement)
 
@@ -400,13 +397,12 @@ class BuildingTool(NavigationTool):
 			for tile in settlement.get_tiles_in_radius(building.position, self._class.radius, include_self=True):
 				obj = tile.object
 				if (obj is not None) and (obj.id in related) and (obj not in self._highlighted_buildings):
-					self._highlighted_buildings.add( (obj, False) ) # False: was_selected, see _restore_highlighted_buildings
+					self._highlighted_buildings.add((obj, False)) # False: was_selected, see _restore_highlighted_buildings
 					# currently same code as highlight_related_buildings
 					inst = obj.fife_instance
 					self.renderer.addOutlined(inst, *self.related_building_outline)
 					self.renderer.addColored(inst, *self.related_building_color)
 
-
 	def _make_surrounding_transparent(self, building):
 		"""Makes the surrounding of building_position transparent and hide buildings
 		that are built upon (tearset)"""
@@ -435,8 +431,8 @@ class BuildingTool(NavigationTool):
 		if settlement is None or not ids: # nothing is related
 			return
 
-		radii = dict((bid, Entities.buildings[bid].radius) for bid in ids)
-		max_radius = max(radii.itervalues())
+		radii = {bid: Entities.buildings[bid].radius for bid in ids}
+		max_radius = max(radii.values())
 
 		for tile in settlement.get_tiles_in_radius(building.position, max_radius, include_self=True):
 			if tile.object is not None and tile.object.id in ids:
@@ -468,7 +464,7 @@ class BuildingTool(NavigationTool):
 				self.renderer.addColored(inst, *self.selectable_comp.selection_color)
 			self.renderer.removeOutlined(inst)
 			modified_tiles.extend(
-			  ( self.session.world.get_tile(point) for point in building.position )
+			  (self.session.world.get_tile(point) for point in building.position)
 			)
 		self._highlighted_buildings.clear()
 		self.highlight_buildable(modified_tiles)
@@ -481,7 +477,7 @@ class BuildingTool(NavigationTool):
 
 	def mouseMoved(self, evt):
 		self.log.debug("BuildingTool mouseMoved")
-		super(BuildingTool, self).mouseMoved(evt)
+		super().mouseMoved(evt)
 		point = self.get_world_location(evt)
 		if self.start_point != point:
 			self.start_point = point
@@ -491,20 +487,20 @@ class BuildingTool(NavigationTool):
 	def mousePressed(self, evt):
 		self.log.debug("BuildingTool mousePressed")
 		if evt.isConsumedByWidgets():
-			super(BuildingTool, self).mousePressed(evt)
+			super().mousePressed(evt)
 			return
 		if evt.getButton() == fife.MouseEvent.RIGHT:
 			self.on_escape()
 		elif evt.getButton() == fife.MouseEvent.LEFT:
 			pass
 		else:
-			super(BuildingTool, self).mousePressed(evt)
+			super().mousePressed(evt)
 			return
 		evt.consume()
 
 	def mouseDragged(self, evt):
 		self.log.debug("BuildingTool mouseDragged")
-		super(BuildingTool, self).mouseDragged(evt)
+		super().mouseDragged(evt)
 		point = self.get_world_location(evt)
 		if self.start_point is not None:
 			self._check_update_preview(point)
@@ -514,7 +510,7 @@ class BuildingTool(NavigationTool):
 		"""Actually build."""
 		self.log.debug("BuildingTool mouseReleased")
 		if evt.isConsumedByWidgets():
-			super(BuildingTool, self).mouseReleased(evt)
+			super().mouseReleased(evt)
 		elif evt.getButton() == fife.MouseEvent.LEFT:
 			point = self.get_world_location(evt)
 
@@ -556,7 +552,7 @@ class BuildingTool(NavigationTool):
 				self.on_escape()
 			evt.consume()
 		elif evt.getButton() != fife.MouseEvent.RIGHT:
-			super(BuildingTool, self).mouseReleased(evt)
+			super().mouseReleased(evt)
 
 	def do_build(self):
 		"""Actually builds the previews
@@ -612,10 +608,10 @@ class BuildingTool(NavigationTool):
 					# check whether to issue a missing res notification
 					# we need the localized resource name here
 					elif building in self.buildings_missing_resources:
-						res_name = self.session.db.get_res_name( self.buildings_missing_resources[building] )
+						res_name = self.session.db.get_res_name(self.buildings_missing_resources[building])
 						self.session.ingame_gui.message_widget.add(
 						  point=building.position.origin,
-						  string_id='NEED_MORE_RES', message_dict={'resource' : res_name})
+						  string_id='NEED_MORE_RES', message_dict={'resource': res_name})
 
 		self.buildings = []
 		self.buildings_action_set_ids = []
@@ -681,7 +677,7 @@ class BuildingTool(NavigationTool):
 		for building in self._related_buildings:
 			# restore selection, removeOutline can destroy it
 			building.get_component(SelectableComponent).set_selection_outline()
-		for fife_instance in self.buildings_fife_instances.itervalues():
+		for fife_instance in self.buildings_fife_instances.values():
 			layer = fife_instance.getLocationRef().getLayer()
 			# layer might not exist, happens for some reason after a build
 			if layer is not None:
@@ -698,7 +694,7 @@ class BuildingTool(NavigationTool):
 					fife_instance.get2dGfxVisual().setTransparency(0)
 				else:
 					# restore regular translucency value, can also be different
-					fife_instance.get2dGfxVisual().setTransparency( BUILDINGS.TRANSPARENCY_VALUE )
+					fife_instance.get2dGfxVisual().setTransparency(BUILDINGS.TRANSPARENCY_VALUE)
 		self._transparencified_instances.clear()
 
 	def _remove_coloring(self):
@@ -710,7 +706,7 @@ class BuildingTool(NavigationTool):
 		self.renderer.removeAllColored()
 
 
-class ShipBuildingToolLogic(object):
+class ShipBuildingToolLogic:
 	"""Helper class to separate the logic needed when building from a ship from
 	the main building tool."""
 
@@ -765,7 +761,7 @@ class ShipBuildingToolLogic(object):
 		pass
 
 
-class SettlementBuildingToolLogic(object):
+class SettlementBuildingToolLogic:
 	"""Helper class to separate the logic needed when building from a settlement
 	from the main building tool"""
 
@@ -795,8 +791,8 @@ class SettlementBuildingToolLogic(object):
 			# Default build on island.
 			for settlement in session.world.settlements:
 				if settlement.owner == player:
-					island = session.world.get_island(Point(*settlement.ground_map.iterkeys().next()))
-					for tile in settlement.ground_map.itervalues():
+					island = session.world.get_island(Point(*next(iter(settlement.ground_map.keys()))))
+					for tile in settlement.ground_map.values():
 						if is_tile_buildable(session, tile, None, island, check_settlement=False):
 							building_tool._color_buildable_tile(tile)
 
@@ -819,8 +815,10 @@ class SettlementBuildingToolLogic(object):
 	# Using messages now.
 	def add_change_listener(self, instance, building_tool):
 		pass
+
 	def remove_change_listener(self, instance, building_tool):
 		pass
+
 	def continue_build(self):
 		pass
 
@@ -828,7 +826,7 @@ class SettlementBuildingToolLogic(object):
 class BuildRelatedBuildingToolLogic(SettlementBuildingToolLogic):
 	"""Same as normal build, except quitting it drops to the build related tab."""
 	def __init__(self, building_tool, instance):
-		super(BuildRelatedBuildingToolLogic, self).__init__(building_tool)
+		super().__init__(building_tool)
 		# instance must be weakref
 		self.instance = instance
 
@@ -837,14 +835,8 @@ class BuildRelatedBuildingToolLogic(SettlementBuildingToolLogic):
 		self.instance().get_component(SelectableComponent).show_menu(jump_to_tabclass=BuildRelatedTab)
 
 	def on_escape(self, session):
-		super(BuildRelatedBuildingToolLogic, self).on_escape(session)
+		super().on_escape(session)
 		self._reshow_tab()
 
 	def continue_build(self):
 		self._reshow_tab()
-
-
-decorators.bind_all(BuildingTool)
-decorators.bind_all(SettlementBuildingToolLogic)
-decorators.bind_all(ShipBuildingToolLogic)
-decorators.bind_all(BuildRelatedBuildingToolLogic)
